/*

  This file is part of HOTSPOTS.DLL source code, see MAIN.C for
  detailed information.

  Copyright (C) 2000-2001 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/


  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************************************************************
****************************************************************************/

#include <windows.h>
#include "../lsapi/lsapi.h"
#include "main.h"

#undef LOGGING

/*
LPSTR strtrimchr(LPSTR str, char chr)
{
	// filter nonsense parameters
	if ( str == NULL || strlen(str) == 0 || chr == '\0' )
		return str;

	UINT left, right;

	left = 0;
	// skip white space from left
	while ( isspace(str[left]) )
		left++;
	// skip char from left
	while ( str[left] == chr )
		left++;

	// hit ending '\0', return empty string
	if ( str[left] == '\0' )
	{
		str[0] = 0;
		return str;
	}

	right = strlen(str)-1;
	// skip whitespace from right
	while ( isspace(str[right]) )
		right--;
	// skip char from right
	while ( (right >= left) && (str[right] == chr) )
		right--;

	// one char string
	if ( left == right )
	{
		str[0] = str[left];
		str[1] = '\0';
		return str;
	}

	memmove(&str[0], &str[left], right - left + 1);
	str[right - left + 1] = 0;

	return str;
}
*/

LPCTSTR trimQuotes(char *buf)
{
	if (strlen(buf) > 1)
	{
		if ( buf[0] == '"' )
			memmove(buf, buf+1, strlen(buf));
		if ( buf[strlen(buf)-1] == '"' )
			buf[strlen(buf)-1] = '\0';
	}

	return buf;
}

// Was there any parsing error?
static BOOL parsing_error;

// Get ULONG number from string, set parsing_error and display
// error message if some error occured.
// Somewhat multi-purpose function, not too healthy C technique...
unsigned long getnum( LPCTSTR str, LPCTSTR hs, LPCTSTR prm )
{
	unsigned long lnum;
	char msgbuf[256];
	char *getnum_err_chr;

	if (str == NULL)
	{
		parsing_error = TRUE;
		return 0;
	}

	getnum_err_chr = NULL;
	lnum = strtoul(str, &getnum_err_chr, 10);

	if ( getnum_err_chr - str < strlen(str) )
	{
		sprintf(msgbuf, "hotspot: \"%s\"\x0Dparameter: \"%s\"\x0Dvalue: \"%s\"\x0D\x0D" "Error at character %d", hs, prm, str, getnum_err_chr - str + 1);
		MessageBox(hDesktop, msgbuf, ST_NAME " parsing error", MB_OK | MB_ICONEXCLAMATION | MB_TOPMOST);
		parsing_error = TRUE;
	}

	return lnum;
}

// Load hotspots from STEP.RC:
BOOL loadConfig( LPCTSTR steprc )
{
	FILE* f;
	char buffer[MAX_LINE_LENGTH];
	char *subpar;

	hotspots_count = 0;
	memset(&hotspots, '\0', sizeof(hotspots));

	parsing_error = FALSE;

	f = LCOpen(steprc);
	if (f)
	{
		char token1[MAX_LINE_LENGTH], token2[MAX_LINE_LENGTH], token3[MAX_LINE_LENGTH], token4[MAX_LINE_LENGTH], extra[MAX_LINE_LENGTH];
		char *tokens[4];
		int count;

		tokens[0] = token1;
		tokens[1] = token2;
		tokens[2] = token3;
		tokens[3] = token4;
		buffer[0] = 0;

		while ( !parsing_error && (hotspots_count < MAX_HOTSPOTS) && LCReadNextConfig(f, "*hs", buffer, sizeof(buffer)) )
		{
			PHOTSPOT ths;

			// allocate new, temporary hotspot with default values
			ths = newHotspot();

			token1[0] = token2[0] = token3[0] = token4[0] = extra[0] = '\0';
			(void) LCTokenize (buffer, tokens, 2, extra);

			// set hotspot name
			trimQuotes(token2);
			ths->name = strdup(token2);

			LOGF("hotspot def start: %s", token2);

			char prm1[MAX_LINE_LENGTH], prm2[MAX_LINE_LENGTH], prm3[MAX_LINE_LENGTH], prm4[MAX_LINE_LENGTH], prmextra[MAX_LINE_LENGTH];
			char* prms[4];
			int prmcount;

			prms[0] = prm1;
			prms[1] = prm2;
			prms[2] = prm3;
			prms[3] = prm4;
			buffer[0] = 0;

			// try to read next hotspot definition lines
			while ( LCReadNextLine(f, buffer, sizeof(buffer)) )
			{
				prm1[0] = prm2[0] = prm3[0] = prm4[0] = prmextra[0] = '\0';

				(void) LCTokenize (buffer, prms, 2, prmextra);

//				strtrim(prm1);

/*				if ( stricmp("*hs", prm1) == 0 )
				{
					LOG("hotspot def incorrectly terminated");
					freeHotspot(ths);
					break;
				}
				else
*/

				// end of hotspot definition, insert it into array:
				if ( stricmp("*hs~", prm1) == 0 )
				{
					LOGF("hotspot def end: %s", ths->name);
					// insert new hotspot into array
					addHotspot(ths);
					// invalidate temporary hotspot pointer
					ths = NULL;
					// continue on to next hotspot definition
					break;
				}
				else
				if ( stricmp("delay", prm2) == 0 )
				{
					LOGF("  prm: delay: %s", prmextra);
					// update temporary hotspot with parameter
					ths->delay = getnum(prmextra, ths->name, "delay");
					if ( parsing_error )
						break;
					ths->timer = ths->delay;
				}
				else
				if ( stricmp("flags", prm2) == 0 )
				{
					trimQuotes(prmextra);
					LOGF("  prm: flags: %s", prmextra);
					// update temporary hotspot with parameter
					setFlags(ths, prmextra);
				}
				else
				if ( stricmp("cursor", prm2) == 0 )
				{
					trimQuotes(prmextra);
					LOGF("  prm: cursor: %s", prmextra);
					// update temporary hotspot with parameter
					ths->cursor = loadCustomCursor(prmextra);
				}
				else
				if ( stricmp("rect", prm2) == 0 )
				{
					LOGF("  prm: rect: %s", prmextra);
					// update temporary hotspot with parameter
					ths->origin.x = getnum(strtok(prmextra, ",: "), ths->name, "origin.x");
					if ( parsing_error )
						break;
					ths->origin.y = getnum(strtok(NULL, ",: "), ths->name, "origin.y");
					if ( parsing_error )
						break;
					ths->size.x = getnum(strtok(NULL, ",: "), ths->name, "size.x");
					if ( parsing_error )
						break;
					ths->size.y = getnum(strtok(NULL, " \0"), ths->name, "size.y");
//					LOGF("width=%d", ths->size.y);
					if ( parsing_error )
						break;
				}
				else
				if ( stricmp("tooltip", prm2) == 0 )
				{
					trimQuotes(prmextra);
					// update temporary hotspot with parameter
					ths->tooltip = strdup(prmextra);
					LOGF("  prm: tooltip: %s", prmextra);
				}
				else
				if ( (stricmp("sound_enter", prm2) == 0) || (stricmp("sound_in", prm2) == 0) )
				{
					trimQuotes(prmextra);
					LOGF("  prm: sound_enter: %s", prmextra);
					// update temporary hotspot with parameter
					ths->sound_enter = strdup(prmextra);
				}
				else
				if ( (stricmp("sound_leave", prm2) == 0) || (stricmp("sound_out", prm2) == 0) )
				{
					trimQuotes(prmextra);
					LOGF("  prm: sound_leave: %s", prmextra);
					// update temporary hotspot with parameter
					ths->sound_leave = strdup(prmextra);
				}
				else
				if ( stricmp("sound_action", prm2) == 0 )
				{
					trimQuotes(prmextra);
					LOGF("  prm: souid_action: %s", prmextra);
					// update temporary hotspot with parameter
					ths->sound_action = strdup(prmextra);
				}
				else
				if ( stricmp("snap_module", prm2) == 0 )
				{
					trimQuotes(prmextra);
					LOGF("  prm: snap_module: %s", prmextra);
					// update temporary hotspot with parameter
					ths->snap_module = strdup(prmextra);
				}
				else
				if ( stricmp("snap_class", prm2) == 0 )
				{
					trimQuotes(prmextra);
					LOGF("  prm: snap_class: %s", prmextra);
					// update temporary hotspot with parameter
					ths->snap_class = strdup(prmextra);
				}
				else
				if ( stricmp("snap_title", prm2) == 0 )
				{
					trimQuotes(prmextra);
					LOGF("  prm: snap_title: %s", prmextra);
					// update temporary hotspot with parameter
					ths->snap_title = strdup(prmextra);
				}
				else
				if ( (stricmp("action_in", prm2) == 0) || (stricmp("action_enter", prm2) == 0) )
				{
					LOGF("  prm: action_in: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_in = strdup(prmextra);
				}
				else
				// action_out parameter
				if ( (stricmp("action_out", prm2) == 0) || (stricmp("action_leave", prm2) == 0) )
				{
					LOGF("  prm: action_out: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_out = strdup(prmextra);
				}
				else
				// action_shift parameter
				if ( stricmp("action_shift", prm2) == 0 )
				{
					LOGF("  prm: action_shift: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_shift = strdup(prmextra);
				}
				else
				// action_alt parameter
				if ( stricmp("action_alt", prm2) == 0 )
				{
					LOGF("  prm: action_alt: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_alt = strdup(prmextra);
				}
				else
				// action_ctrl parameter
				if ( stricmp("action_ctrl", prm2) == 0 )
				{
					LOGF("  prm: action_ctrl: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_ctrl = strdup(prmextra);
				}
				else
				// action_lb parameter
				if ( stricmp("action_lb", prm2) == 0 )
				{
					LOGF("  prm: action_lb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_lb = strdup(prmextra);
				}
				else
				// action_rb parameter
				if ( stricmp("action_rb", prm2) == 0 )
				{
					LOGF("  prm: action_rb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_rb = strdup(prmextra);
				}
				else
				// action_mb parameter
				if ( stricmp("action_mb", prm2) == 0 )
				{
					LOGF("  prm: action_mb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_mb = strdup(prmextra);
				}
				else
				// action_ldb parameter
				if ( stricmp("action_ldb", prm2) == 0 )
				{
					LOGF("  prm: action_ldb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_ldb = strdup(prmextra);
				}
				else
				// action_rdb parameter
				if ( stricmp("action_rdb", prm2) == 0 )
				{
					LOGF("  prm: action_rdb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_rdb = strdup(prmextra);
				}
				else
				// action_mdb parameter
				if ( stricmp("action_mdb", prm2) == 0 )
				{
					LOGF("  prm: action_mdb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_mdb = strdup(prmextra);
				}
				else
				// action_slb parameter
				if ( stricmp("action_slb", prm2) == 0 )
				{
					LOGF("  prm: action_slb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_slb = strdup(prmextra);
				}
				else
				// action_srb parameter
				if ( stricmp("action_srb", prm2) == 0 )
				{
					LOGF("  prm: action_srb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_srb = strdup(prmextra);
				}
				else
				// action_smb parameter
				if ( stricmp("action_smb", prm2) == 0 )
				{
					LOGF("  prm: action_smb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_smb = strdup(prmextra);
				}
				else
				// action_alb parameter
				if ( stricmp("action_alb", prm2) == 0 )
				{
					LOGF("  prm: action_alb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_alb = strdup(prmextra);
				}
				else
				// action_arb parameter
				if ( stricmp("action_arb", prm2) == 0 )
				{
					LOGF("  prm: action_arb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_arb = strdup(prmextra);
				}
				else
				// action_amb parameter
				if ( stricmp("action_amb", prm2) == 0 )
				{
					LOGF("  prm: action_amb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_amb = strdup(prmextra);
				}
				else
				// action_clb parameter
				if ( stricmp("action_clb", prm2) == 0 )
				{
					LOGF("  prm: action_clb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_clb = strdup(prmextra);
				}
				else
				// action_crb parameter
				if ( stricmp("action_crb", prm2) == 0 )
				{
					LOGF("  prm: action_crb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_crb = strdup(prmextra);
				}
				else
				// action_cmb parameter
				if ( stricmp("action_cmb", prm2) == 0 )
				{
					LOGF("  prm: action_cmb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_cmb = strdup(prmextra);
				}
				else
				// action_sldb parameter
				if ( stricmp("action_alb", prm2) == 0 )
				{
					LOGF("  prm: action_sldb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_sldb = strdup(prmextra);
				}
				else
				// action_srdb parameter
				if ( stricmp("action_srdb", prm2) == 0 )
				{
					LOGF("  prm: action_srdb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_srdb = strdup(prmextra);
				}
				else
				// action_smdb parameter
				if ( stricmp("action_smdb", prm2) == 0 )
				{
					LOGF("  prm: action_smdb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_smdb = strdup(prmextra);
				}
				else
				// action_aldb parameter
				if ( stricmp("action_aldb", prm2) == 0 )
				{
					LOGF("  prm: action_aldb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_aldb = strdup(prmextra);
				}
				else
				// action_ardb parameter
				if ( stricmp("action_ardb", prm2) == 0 )
				{
					LOGF("  prm: action_ardb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_ardb = strdup(prmextra);
				}
				else
				// action_amdb parameter
				if ( stricmp("action_amdb", prm2) == 0 )
				{
					LOGF("  prm: action_amdb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_amdb = strdup(prmextra);
				}
				else
				// action_cldb parameter
				if ( stricmp("action_cldb", prm2) == 0 )
				{
					LOGF("  prm: action_cldb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_cldb = strdup(prmextra);
				}
				else
				// action_crdb parameter
				if ( stricmp("action_crdb", prm2) == 0 )
				{
					LOGF("  prm: action_crdb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_crdb = strdup(prmextra);
				}
				else
				// action_cmdb parameter
				if ( stricmp("action_cmdb", prm2) == 0 )
				{
					LOGF("  prm: action_cmdb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_cmdb = strdup(prmextra);
				}
				else
				// action_lrb parameter
				if ( (stricmp("action_lrb", prm2) == 0) || (stricmp("action_rlb", prm2) == 0) )
				{
					LOGF("  prm: action_lrb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_lrb = strdup(prmextra);
				}
				else
				// action_lmb parameter
				if ( (stricmp("action_lmb", prm2) == 0) || (stricmp("action_mlb", prm2) == 0) )
				{
					LOGF("  prm: action_lmb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_lmb = strdup(prmextra);
				}
				else
				// action_mrb parameter
				if ( (stricmp("action_mrb", prm2) == 0) || (stricmp("action_rmb", prm2) == 0) )
				{
					LOGF("  prm: action_mrb: %s", prmextra);
					// update temporary hotspot with parameter
					ths->action_mrb = strdup(prmextra);
				}
			}

			// Shoot-down temporary hotspot, if there was bad / unterminated definition
			if ( ths != NULL )
				freeHotspot(ths);
		}

		LOGF("%d hotspots loaded", hotspots_count);

		LCClose(f);
	}
	else
		return FALSE;

	// Return TRUE, if everything went OK
	return !parsing_error;
}

// Should we support old hotspots configuration format? I don't think so...
BOOL loadOldConfig( LPCTSTR rcpath )
{
	return FALSE;
}

