#include <windows.h>
#include <stdio.h>
#include "main.h"

inline BOOL mouseDrag( void )
{
	return ( LMBUTTON || RMBUTTON || MMBUTTON );
}

//static DWORD save_cursor = 0;
//static HWND save_wnd = NULL;

// Set custom hotspot cursor
void setCustomCursor( PHOTSPOT hs )
{
	if ( hs->default_cursor != NULL )
		return;

	if ( fullScreenWindow(hs) )
		return;

	// Do not change cursor if we are dragging with mouse
	if ( mouseDrag() )
		return;

	LOG("set custom cursor");

	hs->default_cursor = GetCursor();

	// Set all system cursors to our cursor, because we need to change it system-wide
	if ( hs->cursor != NULL )
	{
		for ( UINT i = 0; i < NELEMENTS(syscursornames); i++ )
			SetSystemCursor(hs->cursor, syscursornames[i].id);
		LOGF("set custom cursor: %d", hs->cursor);
	}
}

void setNormalCursor( PHOTSPOT hs )
{
	LOG("set normal cursor");
//	if ( fullScreenWindow(hs) )
//		return;

	for ( UINT i = 0; i < NELEMENTS(syscursornames); i++ )
		if ( STRVALID(syscursornames[i].value) )
			SetSystemCursor(LoadCursorFromFile(syscursornames[i].value), syscursornames[i].id);
		else
			SetSystemCursor(LoadCursor(NULL, MAKEINTRESOURCE(syscursornames[i].res)), syscursornames[i].id);

	if ( hs->default_cursor != NULL )
	{
		LOGF("set default cursor: %d", hs->default_cursor);
		SetCursor(hs->default_cursor);
		hs->default_cursor = NULL;
	}
	else
		LOG("default_cursor is NULL");
}

// Load custom cursor from file/system-resource and set it
HCURSOR loadCustomCursor( LPCTSTR aCursor )
{
	char cursor_path[MAX_PATH+1];

	if ( !STRVALID(aCursor) )
		return NULL;

	// Test if aCursor is any predefined name of system cursor
	for ( UINT i = 0; i < NELEMENTS(cursornames); i++ )
		if ( stricmp(aCursor, cursornames[i].name) == 0 )
			return LoadCursor(NULL, cursornames[i].id);

	cursor_path[0] = 0;
	if ( STRVALID(cursor_dir) )
		strcpy(cursor_path, cursor_dir);
	strcat(cursor_path, aCursor);
	LOGF("Custom cursor: %s", cursor_path);

	return LoadCursorFromFile(cursor_path);
}

// Set system cursors to registry values
void reloadSystemCursors( void )
{
	HKEY hndKey;
	char valbuf[MAX_PATH+1];
	DWORD cb;

	LOG("reloadSystemCursors() called");

	if ( RegOpenKeyEx(HKEY_CURRENT_USER, "Control Panel\\Cursors", 0, KEY_READ, &hndKey) == ERROR_SUCCESS )
	{
		for ( UINT i = 0; i < NELEMENTS(syscursornames); i-- )
			if ( RegQueryValueEx(hndKey, syscursornames[i].name, NULL, NULL, NULL, &cb) == ERROR_SUCCESS )
			{
				RegQueryValueEx(hndKey, syscursornames[i].name, NULL, NULL, valbuf, &cb);
				if (syscursornames[i].value)
					free(syscursornames[i].value);
				syscursornames[i].value = strdup(valbuf);
			}
	}

	LOG("reloadSystemCursors() end");
}

