/*

  This file is part of HOTSPOTS.DLL source code, see MAIN.C for
  detailed information.

  Copyright (C) 2000-2001 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/


  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************************************************************
****************************************************************************/

#include <windows.h>
#include <mmsystem.h>
#include <shellapi.h>
#include "../lsapi/lsapi.h"
#include "main.h"

static volatile BOOL in_timer = FALSE;
static volatile BOOL timer_disabled = FALSE;

int atoid(LPCTSTR str, int def)
{
	return (STRVALID(str) ? atoi(str) : def);
}

BOOL isRelativePath(LPCTSTR path)
{
	if ( strstr(path, ".\\") != NULL )
		return TRUE;

	if ( (strchr(path, '\\') == NULL) && (strchr(path, ':') == NULL) )
		return TRUE;

	return FALSE;
}

void disableTimer(void)
{
	timer_disabled = TRUE;
	// Wait for TimerProc() finish
	while ( in_timer ) {};
}

void enableTimer(void)
{
	timer_disabled = FALSE;
}

// Determine if screensaver is running
BOOL scrSaverRunning( void )
{
	HDESK hDesktop;

	switch (OSVER)
	{
	    // Windows 98 and 95
	    case VER_PLATFORM_WIN32_WINDOWS:
	 		if ( FindWindow("WindowsScreenSaverClass", NULL) != NULL )
				return TRUE;
			break;
	    // Windows NT
		case VER_PLATFORM_WIN32_NT:
			// In Windows NT/2K screen saver acts as another desktop
			hDesktop = OpenDesktop("Screen-saver", 0, FALSE, MAXIMUM_ALLOWED);
			if ( hDesktop == NULL )
				return (GetLastError() == ERROR_ACCESS_DENIED);
			else
			{
				CloseDesktop(hDesktop);
				return TRUE;
			}
			break;
	}

	return FALSE;
}

// Parse action string and execute it
void runParameter( PHOTSPOT hs, LPCTSTR action )
{
	INT flg;
	char aCommand[MAX_LINE_LENGTH], aParams[MAX_LINE_LENGTH];
	char *args[1];

	LOG("run-param enter");
	if ( !STRVALID(action) )
		return;

	if ( !mod_enabled )
		return;

	if ( fullScreenWindow(hs) )
		return;

//	if ( !onAttachedWindow(hs) )
//		return;

	if ( scrSaverRunning() )
		return;

	args[0] = aCommand;
    aCommand[0] = aParams[0] = 0;
	LCTokenize(action, args, 1, aParams);

	if ( aCommand[0] == '!' )
		ParseBangCommand(GetLitestepWnd(), aCommand, aParams);
	else
	{
		if ( aCommand[0] == '@' )
		{
			flg = 0;
			memmove(aCommand, aCommand + 1, strlen(aCommand));
		}
		else
		{
			flg = SW_SHOWDEFAULT;
			if ( hs->flags.maximized )
				flg = SW_SHOWMAXIMIZED;
			if ( hs->flags.minimized )
				flg = SW_SHOWMINIMIZED;
			if ( hs->flags.restore )
				flg = SW_SHOWNA;//RESTORE;
		}

		HWND hwnd;
		RECT mousepos, wndrect;

		if ( hs->flags.one_instance && (hwnd = getHandleFromPath(aCommand)) != NULL )
		{
			LOG("wnd is running");
			if ( hwnd != GetForegroundWindow() )
			{
				LOG("wnd is not in foreground");
				if ( IsIconic(hwnd) ) {
					LOG("wnd is iconic, resore it");
					PostMessage(hwnd, WM_SYSCOMMAND, SC_RESTORE, 0);
				}
				SetForegroundWindow(hwnd);
				if ( window_animation && GetCursorPos((LPPOINT)&mousepos.left) && GetWindowRect(hwnd, &wndrect) )
				{
					mousepos.bottom = mousepos.top;
					mousepos.right = mousepos.left;
					DrawAnimatedRects(GetDesktopWindow(), 0, &mousepos, &wndrect);
				}
			}
		}
		else
			ShellExecute(GetLitestepWnd(), "open", aCommand, aParams, "", flg);
	}
}

// Set hotspot flags from flag string
void setFlags( PHOTSPOT hs, LPCTSTR src )
{
	char *p;
	int i;
	enum {OP_TOGGLE, OP_SET, OP_CLEAR } op;

	if ( (hs == NULL) || !STRVALID(src) )
		return;

	if ( strchr(src, '0') != NULL )
		memset(&hs->flags, '\0', sizeof(FLAGS));

	op	= OP_TOGGLE;
	for ( i = 0; i < strlen(src); i++ )
	{
		switch (src[i])
		{
			case '+':
				op = OP_SET;
				continue;
				break;  // just to be sure ;-)
			case '-':
				op = OP_CLEAR;
				continue;
		}

		switch (src[i])
		{
			case FL_SILENT:
				if ( op == OP_TOGGLE )
					hs->flags.silent = !hs->flags.silent;
				else
					hs->flags.silent = ( op == OP_SET );
				break;
			case FL_DISABLED:
				if ( op == OP_TOGGLE )
					hs->flags.disabled = !hs->flags.disabled;
				else
					hs->flags.disabled = ( op == OP_SET );
				break;
			case FL_MAXIMIZED:
				if ( op == OP_TOGGLE )
					hs->flags.maximized = !hs->flags.maximized;
				else
					hs->flags.maximized = ( op == OP_SET );
				break;
			case FL_MINIMIZED:
				if ( op == OP_TOGGLE )
					hs->flags.minimized = !hs->flags.minimized;
				else
					hs->flags.minimized = ( op == OP_SET );
				break;
			case FL_RESTORE:
				if ( op == OP_TOGGLE )
					hs->flags.restore = !hs->flags.restore;
				else
					hs->flags.restore = ( op == OP_SET );
				break;
			case FL_TOGGLE:
				if ( op == OP_TOGGLE )
					hs->flags.toggle = !hs->flags.toggle;
				else
					hs->flags.toggle = ( op == OP_SET );
				break;
			case FL_IGNORESHIFTS:
				if ( op == OP_TOGGLE )
					hs->flags.ignore_shifts = !hs->flags.ignore_shifts;
				else
					hs->flags.ignore_shifts = ( op == OP_SET );
				break;
			case FL_CONSUMECLICKL:
				if ( op == OP_TOGGLE )
					hs->flags.consume_clickl = !hs->flags.consume_clickl;
				else
					hs->flags.consume_clickl = ( op == OP_SET );
				break;
			case FL_CONSUMECLICKR:
				if ( op == OP_TOGGLE )
					hs->flags.consume_clickr = !hs->flags.consume_clickr;
				else
					hs->flags.consume_clickr = ( op == OP_SET );
				break;
			case FL_CONSUMECLICKM:
				if ( op == OP_TOGGLE )
					hs->flags.consume_clickm = !hs->flags.consume_clickm;
				else
					hs->flags.consume_clickm = ( op == OP_SET );
				break;
			case FL_NODOWNBTN:
				if ( op == OP_TOGGLE )
					hs->flags.no_downbtn = !hs->flags.no_downbtn;
				else
					hs->flags.no_downbtn = ( op == OP_SET );
				break;
			case FL_ONEINSTANCE:
				if ( op == OP_TOGGLE )
					hs->flags.one_instance = !hs->flags.one_instance;
				else
					hs->flags.one_instance = ( op == OP_SET );
				break;
			case FL_ENABLEDINFULLSCREEN:
				if ( op == OP_TOGGLE )
					hs->flags.enabled_in_fullscreen = !hs->flags.enabled_in_fullscreen;
				else
					hs->flags.enabled_in_fullscreen = ( op == OP_SET );
				break;
			case FL_OUTIGNOREDELAY:
				if ( op == OP_TOGGLE )
					hs->flags.out_ignore_delay = !hs->flags.out_ignore_delay;
				else
					hs->flags.out_ignore_delay = ( op == OP_SET );
				break;
		}

		op	= OP_TOGGLE;
	}
}

// Alloc new hotspot and set it to default values
PHOTSPOT newHotspot( void )
{
	PHOTSPOT hs;

	hs = malloc(sizeof(HOTSPOT));
	memset(hs, 0, sizeof(HOTSPOT));

//	hs->sleeping = FALSE;
//	hs->name = NULL;	//strdup(strupper(aName));
//	hs->cursor = NULL;	//LoadCustomCursor(aCursor);
//	hs->def_cursor = NULL;
	hs->delay = def_delay;
	hs->timer = hs->delay;
//	hs->origin.x = x;
//	hs->origin.y = y;
//	hs->size.x = w;
//	hs->size.y = h;
//	hs->class_name = strdup(aClassName);
//	hs->module_name = strdup(aModuleName);
//	hs->docked = (w == 0) && (h == 0) && (STRVALID(aClassName) || STRVALID(aModuleName));
//	hs->action_in = strdup(aActionIn);
//	hs->action_out = NULL;
//	hs->action_shift = NULL;
//	hs->action_alt = NULL;
//	hs->action_ctrl = NULL;
//	hs->action_lb = NULL;
//	hs->action_rb = NULL;
//	hs->action_mb = NULL;
//	hs->action_ldb = NULL;
//	hs->action_rdb = NULL;
//	hs->action_mdb = NULL;
//	hs->action_slb = NULL;
//	hs->action_srb = NULL;
//	hs->action_smb = NULL;
//	hs->action_sldb = NULL;
//	hs->action_srdb = NULL;
//	hs->action_smdb = NULL;
//	hs->action_alb = NULL;
//	hs->action_arb = NULL;
//	hs->action_amb = NULL;
//	hs->action_aldb = NULL;
//	hs->action_ardb = NULL;
//	hs->action_amdb = NULL;
//	hs->action_clb = NULL;
//	hs->action_crb = NULL;
//	hs->action_cmb = NULL;
//	hs->action_cldb = NULL;
//	hs->action_crdb = NULL;
//	hs->action_cmdb = NULL;

	if ( def_flags != NULL )
		setFlags(hs, def_flags);

	return hs;
}

// Dispose hotspot
void freeHotspot( PHOTSPOT hs )
{
	setNormalCursor(hs);

	free(hs->name);
	free(hs->tooltip);
	free(hs->snap_class);
	free(hs->snap_module);
	free(hs->snap_title);
	free(hs->sound_enter);
	free(hs->sound_leave);
	free(hs->sound_action);
	free(hs->action_in);
	free(hs->action_out);
	free(hs->action_shift);
	free(hs->action_alt);
	free(hs->action_ctrl);
	free(hs->action_lb);
	free(hs->action_rb);
	free(hs->action_mb);
	free(hs->action_ldb);
	free(hs->action_rdb);
	free(hs->action_mdb);
	free(hs->action_slb);
	free(hs->action_srb);
	free(hs->action_smb);
	free(hs->action_sldb);
	free(hs->action_srdb);
	free(hs->action_smdb);
	free(hs->action_alb);
	free(hs->action_arb);
	free(hs->action_amb);
	free(hs->action_aldb);
	free(hs->action_ardb);
	free(hs->action_amdb);
	free(hs->action_clb);
	free(hs->action_crb);
	free(hs->action_cmb);
	free(hs->action_cldb);
	free(hs->action_crdb);
	free(hs->action_cmdb);
	free(hs->action_lrb);
	free(hs->action_lmb);
	free(hs->action_mrb);

	DestroyCursor(hs->cursor);

	free(hs);
}

// Add hotspot to list
BOOL addHotspot( PHOTSPOT hs )
{
	if ( hotspots_count < MAX_HOTSPOTS )
	{
		disableTimer();

		hotspots[hotspots_count] = hs;
		hotspots_count++;

		enableTimer();

		return TRUE;
	}
	else
		return FALSE;
}

// Delete hotspot from list
BOOL delHotspot( int pos )
{
	if ( hotspots_count == 0 )
		return FALSE;

	if ( (pos < 0) || (pos >= hotspots_count) )
		return FALSE;

	disableTimer();

	freeHotspot(hotspots[pos]);

	if ( pos < hotspots_count-1 )
		memmove(hotspots[pos], hotspots[pos+1], sizeof(HOTSPOT) * (hotspots_count - 1 - pos));

	if ( last_hotspot == pos )
		last_hotspot = NO_HOTSPOT;

	hotspots_count--;

	enableTimer();

	return TRUE;
}

// Essential function:
// Determine if mouse cursor is inside the hotspot area:

static HWND cached_hwnd = NULL;
static char cached_class[256];
static char cached_mod[MAX_PATH];

BOOL cursorIn( PHOTSPOT hs, POINT *p )
{
	POINT o,s;
	RECT snap_rect;

	HWND wnd;
	char wnd_title[256];
	char fname[_MAX_FNAME], fext[_MAX_EXT];
	POINT mp;

	// If there is any snap window defined, check it first
	if ( STRVALID(hs->snap_module) || STRVALID(hs->snap_class) || STRVALID(hs->snap_title) )
	{
		if ( !GetCursorPos(&mp) )
			return FALSE;
		if ( (wnd = WindowFromPoint(mp)) == NULL)
			return FALSE;

		// Cache class and module names
		if ( wnd != cached_hwnd )
		{
			cached_hwnd = wnd;

			cached_mod[0] = 0;
			_splitpath(getWndFilename(wnd, cached_mod, sizeof(cached_mod)), NULL, NULL, fname, fext);
			strcpy(cached_mod, fname);
			strcat(cached_mod, fext);

			if ( GetClassName(wnd, cached_class, sizeof(cached_class)) == 0 )
				return FALSE;
		}

		if ( STRVALID(hs->snap_module) )
			if ( !match(hs->snap_module, cached_mod) )
				return FALSE;

		if ( STRVALID(hs->snap_class) )
			if ( !match(hs->snap_class, cached_class) )
				return FALSE;

		if ( STRVALID(hs->snap_title) )
		{
			if ( GetWindowText(wnd, wnd_title, sizeof(wnd_title)) == 0 )
				return FALSE;
			if ( !match(hs->snap_title, wnd_title) )
				return FALSE;
		}

		// If we have zero sized hotspot, it is null-sized snap-on-window thingy:
		if ( (hs->size.x == 0) && (hs->size.y == 0) )
		{
//			LOG("snap_win match");
			return TRUE;
		}

		// Set checking bounds to snap window rectangle
		GetWindowRect(wnd, &snap_rect);

//		LOGF("Inside snap window rect: %d", snap_rect.left);
//		LOGF("    %d", snap_rect.top);
//		LOGF("    %d", snap_rect.right);
//		LOGF("    %d", snap_rect.bottom);
	}
	else
		// Set checking bounds to whole screen
		SetRect(&snap_rect, 0, 0, screen_size.x, screen_size.y);

	// We have hotspot's size defined, so determine if we are inside that area:
	if ( hs->origin.x < 0 )
		o.x = snap_rect.right + hs->origin.x;
	else
		o.x = snap_rect.left + hs->origin.x;

	if ( hs->origin.y < 0 )
		o.y = snap_rect.bottom + hs->origin.y;
	else
		o.y = snap_rect.top + hs->origin.y;

	if ( hs->size.x == 0 )
		s.x = snap_rect.right - o.x;
	else
	if ( hs->size.x < 0 )
		s.x = snap_rect.right - o.x + hs->size.x;
	else
		s.x = hs->size.x;

	if ( hs->size.y == 0 )
		s.y = snap_rect.bottom - o.y;
	else
	if ( hs->size.y < 0 )
		s.y = snap_rect.bottom - o.y + hs->size.y;
	else
		s.y = hs->size.y;

	return (p->x >= o.x) && (p->x < o.x + s.x) && (p->y >= o.y) && (p->y < o.y + s.y);
}


// Do click action, return 1 if we have to consume mouse click:
static BOOL was_dblclck = FALSE;

WORD performClickAction( UINT message, WPARAM wParam, LPARAM lParam )
{
	#define		NOBTN	0
	#define		LBTN	1
	#define		RBTN	2
	#define		MBTN	3

	char *act;
	BYTE btn;

	act = NULL;
	btn = NOBTN;

	switch (message)
	{
		case WM_LBUTTONDBLCLK:
		case WM_NCLBUTTONDBLCLK:
			LOG("LDBLCLCK");
			was_dblclck = TRUE;
			btn = LBTN;
			if ( last_hotspot != NO_HOTSPOT )
			{
				if ( SHIFTKEY )
					act = hotspots[last_hotspot]->action_sldb;
				else
				if ( ALTKEY )
					act = hotspots[last_hotspot]->action_aldb;
				else
				if ( CTRLKEY )
					act = hotspots[last_hotspot]->action_cldb;
				else
					act = hotspots[last_hotspot]->action_ldb;
			}
			break;

		case WM_RBUTTONDBLCLK:
		case WM_NCRBUTTONDBLCLK:
			was_dblclck = TRUE;
			btn = RBTN;
			if ( last_hotspot != NO_HOTSPOT )
			{
				if ( SHIFTKEY )
					act = hotspots[last_hotspot]->action_srdb;
				else
				if ( ALTKEY )
					act = hotspots[last_hotspot]->action_ardb;
				else
				if ( CTRLKEY )
					act = hotspots[last_hotspot]->action_crdb;
				else
					act = hotspots[last_hotspot]->action_rdb;
			}
			break;

		case WM_MBUTTONDBLCLK:
		case WM_NCMBUTTONDBLCLK:
			was_dblclck = TRUE;
			btn = MBTN;
			if ( last_hotspot != NO_HOTSPOT )
			{
				if ( SHIFTKEY )
					act = hotspots[last_hotspot]->action_smdb;
				else
				if ( ALTKEY )
					act = hotspots[last_hotspot]->action_amdb;
				else
				if ( CTRLKEY )
					act = hotspots[last_hotspot]->action_cmdb;
				else
					act = hotspots[last_hotspot]->action_mdb;
			}
			break;


		case WM_LBUTTONUP:
		case WM_NCLBUTTONUP:
			LOG("LBUP");
			btn = LBTN;
			if ( was_dblclck )
				was_dblclck = FALSE;
			else
			if ( last_hotspot != NO_HOTSPOT )
			{
				if ( hotspots[last_hotspot]->was_lb || hotspots[last_hotspot]->flags.no_downbtn )
				if ( SHIFTKEY )
					act = hotspots[last_hotspot]->action_slb;
				else
				if ( ALTKEY )
					act = hotspots[last_hotspot]->action_alb;
				else
				if ( CTRLKEY )
					act = hotspots[last_hotspot]->action_clb;
				else
				if ( RMBUTTON )
					act = hotspots[last_hotspot]->action_lrb;
				else
				if ( MMBUTTON )
					act = hotspots[last_hotspot]->action_lmb;
				else
					act = hotspots[last_hotspot]->action_lb;
				hotspots[last_hotspot]->was_lb = FALSE;
			}
			break;

		case WM_RBUTTONUP:
		case WM_NCRBUTTONUP:
			btn = RBTN;
			if ( was_dblclck )
				was_dblclck = FALSE;
			else
			if ( last_hotspot != NO_HOTSPOT )
			{
				if ( hotspots[last_hotspot]->was_rb || hotspots[last_hotspot]->flags.no_downbtn )
				if ( SHIFTKEY )
					act = hotspots[last_hotspot]->action_srb;
				else
				if ( ALTKEY )
					act = hotspots[last_hotspot]->action_arb;
				else
				if ( CTRLKEY )
					act = hotspots[last_hotspot]->action_crb;
				else
				if ( LMBUTTON )
					act = hotspots[last_hotspot]->action_lrb;
				else
				if ( MMBUTTON )
					act = hotspots[last_hotspot]->action_mrb;
				else
					act = hotspots[last_hotspot]->action_rb;
				hotspots[last_hotspot]->was_rb = FALSE;
			}
			break;

		case WM_MBUTTONUP:
		case WM_NCMBUTTONUP:
			btn = MBTN;
			if ( was_dblclck )
				was_dblclck = FALSE;
			else
			if ( last_hotspot != NO_HOTSPOT )
			{
				if ( hotspots[last_hotspot]->was_mb || hotspots[last_hotspot]->flags.no_downbtn )
				if ( SHIFTKEY )
					act = hotspots[last_hotspot]->action_smb;
				else
				if ( ALTKEY )
					act = hotspots[last_hotspot]->action_amb;
				else
				if ( CTRLKEY )
					act = hotspots[last_hotspot]->action_cmb;
				else
				if ( LMBUTTON )
					act = hotspots[last_hotspot]->action_lmb;
				else
				if ( RMBUTTON )
					act = hotspots[last_hotspot]->action_mrb;
				else
					act = hotspots[last_hotspot]->action_mb;
				hotspots[last_hotspot]->was_mb = FALSE;
			}
			break;


		case WM_LBUTTONDOWN:
		case WM_NCLBUTTONDOWN:
			btn = LBTN;
			if ( last_hotspot != NO_HOTSPOT )
				hotspots[last_hotspot]->was_lb = TRUE;
			break;
		case WM_RBUTTONDOWN:
		case WM_NCRBUTTONDOWN:
			btn = RBTN;
			if ( last_hotspot != NO_HOTSPOT )
				hotspots[last_hotspot]->was_rb = TRUE;
			break;
		case WM_MBUTTONDOWN:
		case WM_NCMBUTTONDOWN:
			btn = MBTN;
			if ( last_hotspot != NO_HOTSPOT )
				hotspots[last_hotspot]->was_mb = TRUE;
			break;
	}


	if ( last_hotspot != NO_HOTSPOT )
	{
		if ( STRVALID(act) )
		{
			if ( !hotspots[last_hotspot]->flags.silent )
				if ( STRVALID(hotspots[last_hotspot]->sound_leave) )
					sndPlaySound(hotspots[last_hotspot]->sound_leave, SND_ASYNC | SND_NOSTOP);
				else
				if ( STRVALID(beep_wav) )
					sndPlaySound(beep_wav, SND_ASYNC | SND_NOSTOP);

			runParameter(hotspots[last_hotspot], act);
		}

		// Consume mouse click if requested
		if ( ((btn == LBTN) && (hotspots[last_hotspot]->flags.consume_clickl)) ||
			 ((btn == RBTN) && (hotspots[last_hotspot]->flags.consume_clickr)) ||
			 ((btn == MBTN) && (hotspots[last_hotspot]->flags.consume_clickm)) )
			return 1;
	}

	return 0;
}

// Perform hotspot action, checking shift keys and mouse buttons
void performAction( PHOTSPOT hs, BOOL aOut )
{
	// Do nothing if we are dragging with mouse
	if ( mouseDrag() )
		return;

	// Do we want out-action?
	if ( aOut )
	{
		// Yes, but only if hotspot is sleeping, i.e. in-action has been already triggered
		// or "ignore-delay for out action" flag is set
		if ( !hs->flags.silent )
			if ( STRVALID(hs->sound_leave) )
				sndPlaySound(hs->sound_leave, SND_ASYNC | SND_NOSTOP);
			else
			if ( STRVALID(beep_wav) )
				sndPlaySound(beep_wav, SND_ASYNC | SND_NOSTOP);

		if ( hs->sleeping || hs->flags.out_ignore_delay )
			runParameter(hs, hs->action_out);
	}
	else
	if ( !hs->sleeping )
	{
		hs->sleeping = TRUE;

		if ( !hs->flags.silent )
			if ( STRVALID(hs->sound_enter) )
				sndPlaySound(hs->sound_enter, SND_ASYNC | SND_NOSTOP);
			else
			if ( STRVALID(beep_wav) )
				sndPlaySound(beep_wav, SND_ASYNC | SND_NOSTOP);

		if ( hs->flags.ignore_shifts )
		{
			LOGF("In action: %s", hs->action_in);
			runParameter(hs, hs->action_in);
		}
		else
		//  Shift pressed
		if ( SHIFTKEY )
		{
			LOG("Shift action");
			runParameter(hs, hs->action_shift);
		}
		else
		//  Alt pressed
		if ( ALTKEY )
		{
			LOG("Alt action");
			runParameter(hs, hs->action_alt);
		}
		else
		//  Ctrl pressed
		if ( CTRLKEY )
		{
			LOG("Ctrl action");
			runParameter(hs, hs->action_ctrl);
		}
		else
		{
			LOGF("In action: %s", hs->action_in);
			runParameter(hs, hs->action_in);
		}
	}
}

void resetHotspot( PHOTSPOT hs )
{
	hs->sleeping = FALSE;
	hs->timer = hs->delay;
}

// Heart of module - timer function:
VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT idEvent,	DWORD dwTime )
{
	BOOL found;
	int i;
//	POINT mousepos;

	if ( timer_disabled || in_timer || mouseDrag() )
		return;

//	if ( !GetCursorPos(&mousepos) )
//		return;

	in_timer = TRUE;
	found = FALSE;
	i = hotspots_count;
	while ( !found && (i-- > 0) )
	{
		if ( !hotspots[i]->flags.disabled && cursorIn(hotspots[i], &mousepos) )
		{
			if ( i == last_hotspot )
			{
				if ( !hotspots[i]->sleeping )
					if ( hotspots[i]->timer == 0 )
						performAction(hotspots[i], FALSE);
					else
						hotspots[i]->timer--;
			}
			else
			{
				HWND tmpwnd;

				if ( (tmpwnd = WindowFromPoint(mousepos)) != NULL )
				{
					if ( STRVALID(hotspots[i]->tooltip) )
						addTooltip(tmpwnd, hotspots[i]->tooltip);
//					hotspots[i]->save_wnd = tmpwnd;
//					hotspots[i]->save_wnd_style = GetWindowLong(tmpwnd, GWL_STYLE);
//					if ( SetWindowLong(tmpwnd, GWL_STYLE, hotspots[i]->save_wnd_style | CS_DBLCLKS) == 0 )
//						LOG("error setting wnd style");
				}

				if ( last_hotspot != NO_HOTSPOT )
				{
					LOGF("out hotspot: %s", hotspots[last_hotspot]->name);
//					if ( hotspots[last_hotspot]->save_wnd != NULL )
//						SetWindowLong(hotspots[last_hotspot]->save_wnd, GWL_STYLE, hotspots[last_hotspot]->save_wnd_style);
 					performAction(hotspots[last_hotspot], TRUE);
					resetHotspot(hotspots[last_hotspot]);
					setNormalCursor(hotspots[last_hotspot]);
           		}

				LOGF("in hotspot: %s", hotspots[i]->name);

           		if ( hotspots[i]->delay > 0 )
					hotspots[i]->timer--;
           		else
					performAction(hotspots[i], FALSE);

           		last_hotspot = i;
				setCustomCursor(hotspots[i]);
   			}
			found = TRUE;
		}
	}

	if ( !found && (last_hotspot != NO_HOTSPOT) )
	{
		LOGF("out hotspot: %s", hotspots[last_hotspot]->name);
//		if ( hotspots[last_hotspot]->save_wnd != NULL )
//			SetWindowLong(hotspots[last_hotspot]->save_wnd, GWL_STYLE, hotspots[last_hotspot]->save_wnd_style);
		setNormalCursor(hotspots[last_hotspot]);
		if ( !hotspots[last_hotspot]->flags.disabled )
			performAction(hotspots[last_hotspot], TRUE);
		resetHotspot(hotspots[last_hotspot]);
		last_hotspot = NO_HOTSPOT;

		deleteTooltip();
	}

	in_timer = FALSE;
}


// Enable/disable hotspot
int enableHotspot( LPCTSTR name, BOOL enable )
{
	if ( hotspots_count == 0 )
		return 0;

	if ( !STRVALID(name) )
		return 0;

	int i,c;

	c = 0;

	for ( i = 0; i < hotspots_count; i++ )
	{
		if ( match(name, hotspots[i]->name) )
		if ( hotspots[i]->flags.disabled == enable )
		{
			c++;
			// safer method than just to directly toggle flags->disabled
	        setFlags(hotspots[i], "D");
		}
    }

	return c;
}

// !HotspotUpdate bang function
int updateHotspot( LPCTSTR args )
{
	if ( !STRVALID(args) )
	{
		MessageBox(NULL, "!HotspotUpdate parameters: \n" T_UH_ARGS, ST_NAME, MB_OK | MB_TOPMOST | MB_ICONASTERISK );
		return 0;
	}

	if ( hotspots_count == 0 )
		return 0;

	int i;
	char *tokens[1];
	char name[MAX_LINE_LENGTH], extra[MAX_LINE_LENGTH];
	char *par, *subpar;

	tokens[0] = name;
	name[0] = extra[0] = 0;

	LCTokenize(args, tokens, 1, extra);
	LOGF("name: %s", name);
	LOGF("params: %s", extra);

	for ( i = 0; i < hotspots_count; i++ )
	{
		if ( strcmpi(name, hotspots[i]->name) == 0 )
		{
			par = strtok(extra, "=");

			while (par)
			{
				LOGF("param %s = ", par);
				subpar = strtok(NULL, " ");

				if (subpar)
				{
					LOGF("  %s", subpar);

					disableTimer();

					#define SETSTR(PRM)	\
					if ( strcmpi(par, #PRM) == 0 ) 				\
					{											\
						free(hotspots[i]->PRM);					\
						if ( strcmp(subpar, "\"\"") == 0 )		\
							hotspots[i]->PRM = NULL;			\
						else									\
							hotspots[i]->PRM = strdup(subpar);	\
					}											\
					else

					SETSTR( action_in )		SETSTR( action_out )	SETSTR( action_lb )		SETSTR( action_rb )
					SETSTR( action_mb )		SETSTR( action_slb )	SETSTR( action_srb )	SETSTR( action_smb )
					SETSTR( action_alb )	SETSTR( action_arb )	SETSTR( action_amb )	SETSTR( action_clb )
					SETSTR( action_crb )	SETSTR( action_cmb )	SETSTR( action_ldb )	SETSTR( action_rdb )
					SETSTR( action_mdb )	SETSTR( action_sldb )	SETSTR( action_srdb )	SETSTR( action_smdb )
					SETSTR( action_aldb )	SETSTR( action_amdb )	SETSTR( action_cldb )	SETSTR( action_crdb )
					SETSTR( action_cmdb )	SETSTR( action_lrb )	SETSTR( action_lmb )	SETSTR( action_mrb )
					SETSTR( tooltip )		SETSTR( snap_class )	SETSTR( snap_module )	SETSTR( snap_title )
					SETSTR( sound_enter )	SETSTR( sound_leave )	SETSTR( sound_action )
					if ( strcmpi(par, "rect") == 0 )
					{
						hotspots[i]->origin.x = atoid(strtok(subpar, ",:"), 0);
						LOGF("  origin.x = %d", hotspots[i]->origin.x);
						hotspots[i]->origin.y = atoid(strtok(NULL, ",:"), 0);
						LOGF("  origin.y = %d", hotspots[i]->origin.y);
						hotspots[i]->size.x = atoid(strtok(NULL, ",:"), 0);
						LOGF("  size.x = %d", hotspots[i]->size.x);
						hotspots[i]->size.y = atoid(strtok(NULL, " \0"), 0);
						LOGF("  size.y = %d", hotspots[i]->size.y);
					}
					else
					if ( strcmpi(par, "cursor") == 0 )
					{
						DestroyCursor(hotspots[i]->cursor);
						if ( strcmp(subpar, "\"\"") == 0 )
							hotspots[i]->cursor = NULL;
						else
							hotspots[i]->cursor = loadCustomCursor(subpar);
					}
					else
					if ( strcmpi(par, "flags") == 0 )
						setFlags(hotspots[i], subpar);

					enableTimer();
				}

				par = strtok(NULL, "=");
			}

			break;
		}
	}

	return 1;
}
