/*
   _   _  ___ _____ ____  ____   ___ _____ ____   ____  _     _
  | | | |/ _ \_   _/ ___||  _ \ / _ \_   _/ ___| |  _ \| |   | |
  | |_| | | | || | \___ \| |_) | | | || | \___ \ | | | | |   | |
  |  _  | |_| || |  ___) |  __/| |_| || |  ___) || |_| | |___| |___
  |_| |_|\___/ |_| |____/|_|    \___/ |_| |____(_)____/|_____|_____|


  Loadmodule for Litestep shell (and hereby compatible clones)
  Copyright (C) 2000-2001 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/

  Programmed in LCC-WIN32 free C compiler/tools/IDE by Jacob Navia.
  Lcc-win32 compiler and accompanying tools may be obtained at:
  http://www.cs.virginia.edu/~lcc-win32/

  For another compilers some adjustments/corrections may be needed.


  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************************************************************
31/07/01	Pavel Vitis
	First public release of this code
****************************************************************************/

#include <windows.h>
#include <string.h>
#include <commctrl.h>
#include "../lsapi/lsapi.h"
#include "../kmhook/hookdll.h"
#include "../kmhook/hookevnt.h"
#include "main.h"

static LONG DesktopOldWndProc;

#define TIMER_ID 12944

// Catch-up display and registry changes
LRESULT CALLBACK DesktopProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
 		case WM_DISPLAYCHANGE:
			LOG("DesktopProc() called: WM_DISPLAYCHANGE");
			screen_size.x = LOWORD(lParam);
			screen_size.y = HIWORD(lParam);
			break;
		case WM_SETTINGCHANGE:
			LOGF("DesktopProc() called: WM_SETTINGCHANGE: %s", (LPCTSTR) lParam);
			if ( ((LPCTSTR) lParam != NULL) && (stricmp((LPCTSTR) lParam, "windows") == 0) )
				reloadSystemCursors();
			break;
	}

	return CallWindowProc((WNDPROC) DesktopOldWndProc, hwnd, message, wParam, lParam);
}

void setDesktopHook( void )
{
	LOG("setDesktopHook() called");
	DesktopOldWndProc = GetWindowLong(hDesktop, GWL_WNDPROC);
	SetWindowLong(hDesktop, GWL_WNDPROC, (LONG) DesktopProc);
	LOG("setDesktopHook() end");
}

void unsetDesktopHook( void )
{
	LOG("unsetDesktopHook() called");
	SetWindowLong(hDesktop, GWL_WNDPROC, DesktopOldWndProc);
	LOG("unsetDesktopHook() end");
}


__declspec( dllexport ) int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	OSVERSIONINFO osvinfo;

	char buffer[MAX_LINE_LENGTH];
	char rcpath[MAX_PATH+1];

	LOG("initModuleEx() called");

	// If no mouse present, exit with OK, but do nothing.
	// Silent module unload at this moment would be better.
	LOG("GetSystemMetrics(SM_MOUSEPRESENT) called");
	if (GetSystemMetrics(SM_MOUSEPRESENT) == 0)
	{
		LOG("Mouse not present: exiting: 0");
		return 0;
	}

	// Remember dll instance
	DLLInstance = dllInst;
	// Get double-click delay, we will service double clicks ourselves
	DoubleClickTime = GetDoubleClickTime();
	// Cache screen sizes
	LOG("Getting screen size");
	screen_size.x = GetSystemMetrics(SM_CXSCREEN);
	screen_size.y = GetSystemMetrics(SM_CYSCREEN);

	// Get Windows version
	LOG("GetVersionInfo() called");
	osvinfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if ( !GetVersionEx(&osvinfo) )
	{
		LOG("Error obtaining version info: last error = ");
		LOGLE;
		LOG("Exiting: 1");
		return 1;
	}
	OSVER = osvinfo.dwPlatformId;

	// Cache HWND of Litestep and Desktop windows
	LOG("GetLitestepWnd() called");
	hLitestep = GetLitestepWnd();
	hDesktop = FindWindow("DesktopBackgroundClass", NULL);
	if (!hDesktop)
	{
		LOG("No LS desktop, GetDesktopWindow() called");
		hDesktop = GetDesktopWindow();
	}

	// Try to load system-hook dll
	LOG("loadHookDLL() called");
	if ( !loadHookDLL( dllInst ) )
	{
		LOG("Error: Exiting: 1");
		return 1;
	}

	// Create tool window
	LOG("createToolWindow() called");
	if ( !createToolWindow() )
	{
		LOG("Error: Exiting: 1");
		return 1;
	}

	// Create tooltips window
	LOG("createTooltipsWindow() called");
	if ( !createTooltipsWindow() )
	{
		LOG("Error: Exiting: 1");
		return 1;
	}

	// Get some variables from step.rc
	LOG("Getting module variables from STEP.RC");
	mod_enabled = GetRCBool("HotspotsDisabled", FALSE);
	disabled_in_fullscreen = GetRCBool("HotspotsDisabledInFullScreen", TRUE);
	window_animation = GetRCBool("HotspotsWindowAnimation", TRUE);
	def_delay = 0;

	// Get another variables
	LOG("Getting HotspotCursorDir");
	buffer[0] = 0;
	if ( GetRCString("HotspotCursorDir", buffer, "", sizeof(buffer)-1) )
	{
		if ( (strlen(buffer) > 0) && (buffer[strlen(buffer)-1] != '\\') )
			strcat(buffer, "\\");
		cursor_dir = strdup(buffer);
	}
	else
		cursor_dir = NULL;

	LOG("Getting HotspotFlags");
	buffer[0] = 0;
	if ( GetRCString("HotspotFlags", buffer, "", sizeof(buffer)-1) )
		def_flags = strdup(buffer);
	else
		def_flags = NULL;

	LOG("Getting STEP.RC path");
	strcpy(rcpath, szPath);
	if ( rcpath[strlen(rcpath)-1] != '\\' )
		strcat(rcpath, "\\");
	strcat(rcpath, "step.rc");

	// Load hotspots definitions from step.rc
	if ( GetRCBool("HotspotsOldConfig", TRUE) )
	{
		if ( !loadOldConfig(rcpath) )
		{
			free(cursor_dir);
			free(def_flags);
			return 1;
		}
	}
	else
	{
		LOG("loadConfig() called");
		if ( !loadConfig(rcpath) )
		{
			free(cursor_dir);
			free(def_flags);
			LOG("Error: Exiting: 1");
			return 1;
		}
	}
	LOG("Config loaded");


	LOG("Loading system cursors");
	reloadSystemCursors();

	buffer[0] = 0;
	if ( GetRCString("HotspotBeep", buffer, "", sizeof(buffer)-1) )
	{
		if ( (strlen(buffer) > 0) && (buffer[strlen(buffer)-1] != '\\') )
			strcat(buffer, "\\");
		beep_wav = strdup(buffer);
	}
	else
		beep_wav = NULL;

	// Set mouse system-wide hook
	LOG("setMouseHook() called");
	(setMouseHook)(tool_window, EV_MOUSEUPS | EV_MOUSEDOWNS | EV_NCMOUSEUPS | EV_NCMOUSEDOWNS | EV_MOUSEMOVE | EV_NCMOUSEMOVE );
	LOG("Mouse hook set");

	// Hook desktop procedure
	setDesktopHook();
	LOG("Desktop hook set");

	// Add bang commands
	LOG("adding bangs");
	addBangs();

	last_hotspot = NO_HOTSPOT;
	GetCursorPos(&mousepos);

	// Finally, crank the engine...
	LOG("Initializing timer");
//	hotspot_timer = SetTimer(tool_window, TIMER_ID, 10, NULL);
	hotspot_timer = SetTimer(NULL, 0, TIMER_RES, TimerProc);

	LOG("Init done OK");
	return 0;
}


__declspec( dllexport ) void quitModule(HINSTANCE dllInst)
{
	LOG("quitModule() called");

//	KillTimer(tool_window, TIMER_ID);
	LOG("Killing timer");
	KillTimer(NULL, hotspot_timer);

  	unsetDesktopHook();

	LOG("unsetMouseHook() called");
	(unsetMouseHook)(tool_window);

	LOG("unloadHookDLL() called");
	unloadHookDLL();

	LOG("Destroying tool window");
	destroyToolWindow();
	LOG("Destroying tooltips window");
	destroyTooltipsWindow();

	LOG("Freeing hotspots");
	for ( int i = 0; i < hotspots_count; i++ )
		freeHotspot(hotspots[i]);

	LOG("Freeing system cursor names");
	for ( int i = 0; i < NELEMENTS(syscursornames); i++ )
		free(syscursornames[i].value);

	LOG("Removing bangs");
	removeBangs();

	LOG("Freeing other heap data");
	free(beep_wav);
	free(cursor_dir);
	free(def_flags);

	LOG("Quit done OK");
}

__declspec( dllexport ) int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd)
{
	return initModuleEx (ParentWnd, dllInst, wd->lsPath);
}


__declspec( dllexport ) BOOL WINAPI LibMain(HINSTANCE hDLLInst, DWORD fdwReason, LPVOID lpvReserved)
{
	UNREFERENCED_PARAMETER(lpvReserved);

	if (fdwReason == DLL_PROCESS_ATTACH)
	{
		LOG("DllMain called: DLL_PROCESS_ATTACH");
		DisableThreadLibraryCalls(hDLLInst);
	}
	return TRUE;
}

