/*

  This file is part of HOTSPOTS.DLL source code, see MAIN.C for
  detailed information.

  Copyright (C) 2000-2001 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/


  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************************************************************
****************************************************************************/

#ifndef _MAIN_H_
#define _MAIN_H_

#include <windows.h>


#define LOGGING

#ifdef LOGGING
	char logmsg[256];
	LPVOID lpMsgBuf;

	#define LOG(TXT)		OutputDebugString(ST_NAME TEXT(": ") TXT TEXT("\n"))
	#define LOGF(TXT,PRM)	sprintf(logmsg, ST_NAME TEXT(": ") TXT TEXT("\n"), PRM); \
							OutputDebugString(logmsg)
	#define LOGIF(IF,TXT)	if (IF)	OutputDebugString(ST_NAME TEXT(": ") TXT TEXT("\n"))
	#define LOGLE			FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(), \
							MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0, NULL); \
							OutputDebugString(lpMsgBuf); \
							LocalFree( lpMsgBuf )
#else
	#define LOG(TXT)
	#define LOGF(TXT,PRM)
	#define LOGIF(IF,TXT)
	#define LOGLE
#endif



#define ST_SHORTNAME	TEXT("hotspots")
#define ST_NAME			TEXT("HOTSPOTS.DLL")
#define ST_VERSION		TEXT("0.60")
#define ST_AUTHOR		TEXT("pavelvitis")
#define ST_REVID		ST_SHORTNAME TEXT(" ") ST_VERSION TEXT(" (") __DATE__ TEXT(" / ") __TIME__ TEXT(") ") ST_AUTHOR

// window classes
#define WC_LSDESKTOP	TEXT("DesktopBackgroundClass")
#define WC_MAIN			TEXT("LS_") ST_SHORTNAME

#define T_UH_ARGS		TEXT("name [flags=\"flags\"] [cursor=\"cursor\"] [action_type=\"action\"] [tooltip=\"text\"]")

#define NELEMENTS(array) (sizeof(array)/sizeof(array[0]))
#define STRVALID(s) ((s != NULL) && (strlen(s) > 0))

#define NO_HOTSPOT -1
#define MAX_HOTSPOTS 100

// Check mouse position every 10 msec
#define TIMER_RES	10

//#define memChunkSize 10

// Known hotspot flags are defined here:
#define FL_DISABLED				'D'
#define FL_SILENT				'S'
#define FL_MAXIMIZED			'X'
#define FL_MINIMIZED			'M'
#define FL_RESTORE				'R'
#define FL_TOGGLE				'T'
#define FL_CONSUMECLICKL		'1'
#define FL_CONSUMECLICKR		'2'
#define FL_CONSUMECLICKM		'3'
#define FL_NODOWNBTN			'B'
#define FL_ONEINSTANCE			'O'
#define FL_IGNORESHIFTS			'I'
#define FL_ENABLEDINFULLSCREEN	'F'
#define FL_OUTIGNOREDELAY		'U'

// Structure for hotspot flags:
typedef struct
{
	BOOL disabled, silent, maximized, minimized, restore, toggle;
	BOOL ignore_shifts, consume_clickl, consume_clickr, consume_clickm, no_downbtn;
	BOOL one_instance, enabled_in_fullscreen, out_ignore_delay;
} FLAGS, *PFLAGS;

// Hotspot structure:
typedef struct tagHOTSPOT
{
	// Hotspot's name:
	LPSTR name;
	POINT origin;
	POINT size;
	BOOL sleeping;
	BOOL docked;
	BOOL was_lb, was_rb, was_mb;
	UINT delay;
	UINT timer;
	FLAGS flags;
	HCURSOR cursor, default_cursor;
	LPSTR tooltip;
//	HWND save_wnd;
//	LONG save_wnd_style;
	LPSTR snap_class, snap_title, snap_module;
	LPSTR sound_enter, sound_leave, sound_action;
	// Every action has its own string:
	LPSTR action_in, action_out, action_shift, action_alt, action_ctrl;
	LPSTR action_lb, action_rb, action_mb;
	LPSTR action_ldb, action_rdb, action_mdb;
	LPSTR action_slb, action_srb, action_smb;
	LPSTR action_sldb, action_srdb, action_smdb;
	LPSTR action_alb, action_arb, action_amb;
	LPSTR action_aldb, action_ardb, action_amdb;
	LPSTR action_clb, action_crb, action_cmb;
	LPSTR action_cldb, action_crdb, action_cmdb;
	LPSTR action_lrb, action_lmb, action_mrb;
//	struct tagHOTSPOT *next;
} HOTSPOT, *PHOTSPOT;

typedef struct {
	LPSTR name;
	LPTSTR id;
} CURSORNAME;

typedef struct {
	LPSTR name;
	LPTSTR res;
	DWORD id;
	LPSTR value;
} SYSCURSORNAME;

#define IDC_SIZE			IDC_SIZEALL
#define IDC_HAND            MAKEINTRESOURCE(32649)
#define OCR_HAND            32649

#define ALTKEY ((GetAsyncKeyState(VK_MENU) & 0x8000U) != 0)
#define CTRLKEY ((GetAsyncKeyState(VK_CONTROL) & 0x8000U) != 0)
#define LSHIFTKEY ((GetAsyncKeyState(VK_LSHIFT) & 0x8000U) != 0)
#define RSHIFTKEY ((GetAsyncKeyState(VK_RSHIFT) & 0x8000U) != 0)
#define SHIFTKEY ((GetAsyncKeyState(VK_SHIFT) & 0x8000U) != 0)

#define LMBUTTON ((GetAsyncKeyState(VK_LBUTTON) & 0x8000U) != 0)
#define RMBUTTON ((GetAsyncKeyState(VK_RBUTTON) & 0x8000U) != 0)
#define MMBUTTON ((GetAsyncKeyState(VK_MBUTTON) & 0x8000U) != 0)

#define SYSCURNAMES_CNT 13
#define CURNAMES_CNT 30
extern SYSCURSORNAME syscursornames[SYSCURNAMES_CNT];
extern CURSORNAME cursornames[CURNAMES_CNT];


// Declarations of global variables:
//extern WORD memChunkCount;
extern volatile POINT mousepos;
extern DWORD def_delay;
extern LPSTR def_flags;
extern BOOL mod_enabled;
extern BOOL window_animation;
extern BOOL disabled_in_fullscreen;
extern int last_hotspot;

extern DWORD DoubleClickTime;
extern WORD hotspot_timer;
extern POINT screen_size;
extern WORD hotspots_count;
extern DWORD OSVER;
extern HWND tool_window;
extern HWND tooltips_window;
extern HWND hLitestep;
extern HWND hDesktop;
extern HINSTANCE DLLInstance;
extern LPSTR cursor_dir;
extern LPSTR beep_wav;
extern DWORD OSVER;

extern PHOTSPOT hotspots[MAX_HOTSPOTS];

// Function declarations:
//void timerProc();
void CALLBACK TimerProc( HWND, UINT, UINT, DWORD );
WORD performClickAction( UINT, WPARAM, LPARAM );
BOOL createToolWindow( void );
BOOL createTooltipsWindow( void );
void destroyToolWindow( void );
void destroyTooltipsWindow( void );
BOOL loadConfig( LPCTSTR );
BOOL loadOldConfig( LPCTSTR );
void freeHotspot( PHOTSPOT );
BOOL fullScreenWindow( PHOTSPOT );
HWND getHandleFromPath( LPCTSTR );
LPCTSTR getWndFilename( HWND, LPSTR, int  );
BOOL onAttachedWindow( PHOTSPOT );
PHOTSPOT newHotspot( void );
void freeHotspot( PHOTSPOT );
HCURSOR loadCustomCursor( LPCTSTR );
void setFlags( PHOTSPOT, LPCTSTR );
int enableHotspot( LPCTSTR, BOOL );
int updateHotspot( LPCTSTR );
BOOL addHotspot( PHOTSPOT );
BOOL delHotspot( int );
//BOOL delHotspot( LPCTSTR );
char *strdup(const char *string);

void addBangs(void);
void removeBangs(void);
void bangModEnable(HWND, char* );
void bangModDisable(HWND, char* );
void bangModToggle(HWND, char* );
void bangHotspotUpdate(HWND, char* );
void bangHotspotEnable(HWND, char* );
void bangHotspotDisable(HWND, char* );

void addTooltip(HWND, LPSTR);
void deleteTooltip(void);

int GetWndFilenamePDH( DWORD, LPSTR, int );

BOOL mouseDrag( void );
void setCustomCursor( PHOTSPOT );
void setNormalCursor( PHOTSPOT );
HCURSOR loadCustomCursor( LPCTSTR );
void reloadSystemCursors( void );

#endif // _MAIN_H_

