#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <pdh.h>
#include "main.h"

#define Key "SOFTWARE\\Microsoft\\WindowsNT\\CurrentVersion\\Perflib\\009"

void GetIndex( char *, char * );

/********************************************************************\
* Function: void DisplayFilename( DWORD )                            *
*                                                                    *
* Purpose : Display executable filename of the process whose PID     *
*           is passed in as a parameter.                             *
*                                                                    *
* Comment : The information is retrieved from the performance        *
*           data in the registry.                                    *
*                                                                    *
\********************************************************************/

int GetWndFilenamePDH( DWORD dwProcessId, LPSTR fname, int maxcount )
{
	DWORD                     CurrentProcessId;
	BOOL                      bContinue = TRUE;
	char                      szIndex[256] = "";
	DWORD                     dwBytes = 12000;
	DWORD                     dwProcessIdOffset;

	int                       i;

	PPERF_DATA_BLOCK          pdb;
	PPERF_OBJECT_TYPE         pot;
	PPERF_INSTANCE_DEFINITION pid;
	PPERF_COUNTER_BLOCK       pcb;
	PPERF_COUNTER_DEFINITION  pcd;

	fname[0] = '\0';

	// Get the index for the PROCESS object.
	GetIndex( "Process", szIndex );

	// Get memory for PPERF_DATA_BLOCK.
	pdb = (PPERF_DATA_BLOCK) HeapAlloc( GetProcessHeap(),
                                          HEAP_ZERO_MEMORY,
                                          dwBytes);

	// Get performance data.
	while( RegQueryValueEx(HKEY_PERFORMANCE_DATA, (LPTSTR)szIndex, NULL, NULL, (LPBYTE)pdb, &dwBytes) == ERROR_MORE_DATA )
	{
		// Increase memory.
		dwBytes += 1000;

		// Allocated memory is too small; reallocate new memory.
		pdb = (PPERF_DATA_BLOCK) HeapReAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, (LPVOID)pdb, dwBytes);
	}

	// Get PERF_OBJECT_TYPE.
	pot = (PPERF_OBJECT_TYPE)((PBYTE)pdb + pdb->HeaderLength);

	// Get the first counter definition.
	pcd = (PPERF_COUNTER_DEFINITION)((PBYTE)pot + pot->HeaderLength);

	// Get index value for ID_PROCESS.
	szIndex[0] = '\0';
	GetIndex( "ID Process", szIndex );

	for( i=0; i< (int)pot->NumCounters; i++ )
	{
		if (pcd->CounterNameTitleIndex == (DWORD)atoi(szIndex))
		{
			dwProcessIdOffset = pcd->CounterOffset;
			break;
		}

		pcd = ((PPERF_COUNTER_DEFINITION)((PBYTE)pcd + pcd->ByteLength));
	}

	// Get the first instance of the object.
	pid = (PPERF_INSTANCE_DEFINITION)((PBYTE)pot + pot->DefinitionLength);

	// Get the name of the first process.
	pcb = (PPERF_COUNTER_BLOCK) ((PBYTE)pid + pid->ByteLength );
	CurrentProcessId = *((DWORD *) ((PBYTE)pcb + dwProcessIdOffset));

	// Find the process object for PID passed in, then print its filename.

	for( i = 1; i < pot->NumInstances && bContinue; i++ )
	{
		if( CurrentProcessId == dwProcessId )
		{
			strncpy(fname, (char *) ((PBYTE)pid + pid->NameOffset), maxcount );
//			printf( "The filename is %ls.exe.\n", (char *) ((PBYTE)pid + pid->NameOffset) );
			bContinue = FALSE;
		}
		else
		{
			pid = (PPERF_INSTANCE_DEFINITION) ((PBYTE)pcb + pcb->ByteLength);
			pcb = (PPERF_COUNTER_BLOCK) ((PBYTE)pid + pid->ByteLength);
			CurrentProcessId = *((DWORD *)((PBYTE)pcb + dwProcessIdOffset));
		}
	}

//	if( bContinue == TRUE )
//		printf( "Not found.\b" );

	// Free the allocated memory.
//	if( !HeapFree(GetProcessHeap(), 0, (LPVOID)pdb) )
//		printf( "HeapFree failed in main.\n" );

	// Close handle to the key.
	RegCloseKey( HKEY_PERFORMANCE_DATA );

	return strlen(fname);
}

/********************************************************************\
* Function: void GetIndex( char *, char * )                          *
*                                                                    *
* Purpose : Get the index for the given counter                      *
*                                                                    *
* Comment : The index is returned in the parameter szIndex           *
*                                                                    *
\********************************************************************/

void GetIndex( char *pszCounter, char *szIndex )
{
	char*  pszBuffer;
	char*  pszTemp;
	char   szObject[256] = "";
	DWORD  dwBytes;
	HANDLE hKeyIndex;
	int    i = 0;
	int    j = 0;

	// Open the key.
	RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                    Key,
                    0, KEY_READ,
                    &hKeyIndex );

	// Get the size of the counter.
	RegQueryValueEx( hKeyIndex,
                       "Counters",
                       NULL, NULL, NULL,

                       &dwBytes );

	// Allocate memory for the buffer.
	pszBuffer = (char *) HeapAlloc( GetProcessHeap(),
                                      HEAP_ZERO_MEMORY,
                                      dwBytes );

	// Get the titles and counters.
	RegQueryValueEx( hKeyIndex,
                       "Counters",
                       NULL, NULL,
                       (LPBYTE)pszBuffer,
                       &dwBytes );

	// Find the index value for PROCESS.
	pszTemp = pszBuffer;

	while( i != (int)dwBytes )
	{
		while (*(pszTemp+i) != '\0')
		{
			szIndex[j] = *(pszTemp+i);
			i++;
			j++;
		}
		szIndex[j] = '\0';
		i++;
		j = 0;
		while (*(pszTemp+i) != '\0')
		{
			szObject[j] = *(pszTemp+i);
			i++;
			j++;
		}
		szObject[j] = '\0';
		i++;
		j = 0;
		if( *(pszTemp+i) == '\0' )
			i++;

		if( strcmp(szObject, pszCounter) == 0 )
			break;
	}

	// Deallocate the memory.
	HeapFree( GetProcessHeap(), 0, (LPVOID)pszBuffer );

	// Close the key.
	RegCloseKey( hKeyIndex );
}

