/*

  This file is part of HOTSPOTS.DLL source code, see MAIN.C for
  detailed information.

  Copyright (C) 2000-2001 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/


  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************************************************************************
****************************************************************************/

#include <windows.h>
#include <tlhelp32.h>
#include <commctrl.h>
#include "../lsapi/lsapi.h"
#include "main.h"

// Get module name from HWND - uses ToolHelp32 (Win98), may not work on WinNT
// probably need some adjustments
LPCTSTR getWndFilename( HWND wnd, LPSTR fname, int maxcount  )
{
	DWORD hid;
	HANDLE snap;
//	char szp[MAX_PATH+1];
	PROCESSENTRY32 data;

	if ( wnd == NULL )
		return NULL;

//	if (!IsWindowVisible(wnd))
//		return TRUE;// Not visible

//	if (!SendMessage(wnd, WM_GETTEXT, sizeof(String), (LPARAM)String))
//		return TRUE;// No window title

	if (GetWindowThreadProcessId(wnd, &hid) == 0)
		return NULL;

//	GetWndFilenamePDH( hid, fname, maxcount );
//	LOGF("PDH filename: \"%s\"", fname);

	snap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
	data.dwSize = sizeof(data);
	if (Process32First(snap, &data))
		do {
			if (data.th32ProcessID == hid)
			{
				CloseHandle(snap);
				strncpy(fname, data.szExeFile, maxcount);
				// Make sure that it is terminated
				fname[maxcount-1] = 0;
				return fname;
			}
		} while (Process32Next(snap, &data));

	CloseHandle(snap);

	return NULL;
}

// Determine if window is application main window
BOOL isAppWindow( HWND wnd )
{
	DWORD nStyle;
	HWND hOwner, hParent;
	BOOL bResult;

	bResult = TRUE;

	nStyle = GetWindowLong( wnd, GWL_STYLE );
	if ( ((nStyle & WS_CHILD) != 0) || ((nStyle & WS_VISIBLE) == 0) )
		bResult = FALSE;

	nStyle = GetWindowLong( wnd, GWL_EXSTYLE );
	if ( (nStyle & WS_EX_TOOLWINDOW) != 0 )
		bResult = FALSE;

	hOwner = GetWindow( wnd, GW_OWNER );
	if ( (hOwner != NULL) && bResult )
		bResult = !isAppWindow( hOwner );

	hParent = GetParent( wnd );
	if ( hParent != NULL )
		bResult = FALSE;

	nStyle = GetWindowLong( wnd, GWL_USERDATA );
	if ( nStyle == magicDWord )
		bResult = FALSE;

	return bResult;
}

// Try to determine if the cursor is on fullscreen window.
// It is somewhat unreliable in that stage:
BOOL fullScreenWindow( PHOTSPOT hs )
{
	RECT wr;
	HWND fwin;

	if ( !disabled_in_fullscreen )
		return FALSE;

	if ( hs->flags.enabled_in_fullscreen )
		return FALSE;

	fwin = GetForegroundWindow();
	GetWindowRect(fwin, &wr);
	if ((wr.left <= 0) && (wr.top <= 0) && (wr.right >= screen_size.x) && (wr.bottom >= screen_size.y))
	{
//    GetClassName(ForegroundWin, ClassName, sizeof(ClassName));
//    if SearchFSExclusionList(ClassName) then
//      result:=false
//    else
		return TRUE;
	}

	return FALSE;
}

// Determine if mouse cursor is on hotspot's snap-window, so we can
// "cover" that window with this hotspot
/*
BOOL onAttachedWindow( PHOTSPOT hs )
{
	HWND wnd;
	char aClassName[256];
	char aModuleName[MAX_PATH+1];
	char fname[_MAX_FNAME], fext[_MAX_EXT];
	POINT mp;

	if ( !STRVALID(hs->snap_class) && !STRVALID(hs->module_name) )
		return TRUE;

	if ( !GetCursorPos(&mp) )
		return FALSE;

	if ( (wnd = WindowFromPoint(mp)) == NULL)
		return FALSE;

	aClassName[0] = aModuleName[0] = 0;
	if ( GetClassName(wnd, aClassName, sizeof(aClassName)) == 0 )
		return FALSE;

	_splitpath(getWndFilename(aModuleName, wnd), NULL, NULL, fname, fext);
	strcpy(aModuleName, fname);
	strcat(aModuleName, fext);

	if ( STRVALID(hs->class_name) && !match(hs->class_name, aClassName) )
		return FALSE;

	if ( STRVALID(hs->module_name) && !match(hs->module_name, aModuleName) )
		return FALSE;

	return TRUE;
}
*/

typedef struct {
	LPCTSTR exePath;
	HWND hwnd;
} PARAM, *PPARAM;

static PARAM aParam;

BOOL CALLBACK checkPaths(HWND hwnd,	LPARAM lParam)
{
	if ( !isAppWindow(hwnd) )
		return TRUE;

    char wpath[MAX_PATH+1];

	if ( getWndFilename( hwnd, wpath, sizeof(wpath) ) == NULL )
		return TRUE;

	if ( stricmp(wpath, ((PPARAM) lParam)->exePath ) == 0 )
	{
		((PPARAM) lParam)->hwnd = hwnd;
		return FALSE;
	}

	return TRUE;
}

HWND getHandleFromPath( LPCTSTR exePath )
{
	aParam.exePath = exePath;
	aParam.hwnd = NULL;
	EnumWindows( &checkPaths, (LPARAM) &aParam );

	return aParam.hwnd;
}


static HWND last_tooltip = NULL;

// Delete last defined tool from tooltip window -
// - we have always only one tooltip at a time
void deleteTooltip(void)
{
	if ( !tooltips_window )
		return;

	if ( !last_tooltip )
		return;

	LOGF("delete tooltip: \"%d\"", last_tooltip);

	TOOLINFO ti;    // tool information

	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = last_tooltip;
	ti.uId = (UINT) last_tooltip;

	SendMessage(tooltips_window, TTM_DELTOOL, 0, (LPARAM) (LPTOOLINFO) &ti);

	last_tooltip = NULL;
}

// Add tool into tooltip window
void addTooltip(HWND wnd, LPSTR text)
{
	deleteTooltip();

	if ( !tooltips_window || !wnd || !STRVALID(text) )
		return;

	LOGF("add tooltip: \"%s\"", text);
	LOGF("add tooltip wnd: \"%d\"", wnd);

	TOOLINFO ti;    // tool information
	RECT r;
//	int id = 0;     // offset to string identifiers
//    static char *szTip[1] = {"TESTING HINT"};

	GetClientRect(wnd, &r);

	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = TTF_IDISHWND | TTF_SUBCLASS;
	ti.hwnd = wnd;
	ti.hinst = DLLInstance;
	ti.uId = (UINT) wnd;
	ti.lpszText = text;
	ti.rect = r;

	SendMessage(tooltips_window, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) &ti);
	SetWindowPos(tooltips_window, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOSENDCHANGING);

	last_tooltip = wnd;
}

// Create window for serving tooltips
BOOL createTooltipsWindow(void)
{
	InitCommonControls();

	tooltips_window = CreateWindow(
			TOOLTIPS_CLASS,
			ST_SHORTNAME "_hints",
			TTS_ALWAYSTIP,
			CW_USEDEFAULT,
			CW_USEDEFAULT,
			CW_USEDEFAULT,
			CW_USEDEFAULT,
			NULL,
			(HMENU) NULL,
			DLLInstance,
			NULL
			);

	if ( !tooltips_window )
	{
		MessageBox(NULL, "Error creating hints window", ST_NAME, MB_OK);
		return FALSE;
	}

	SetWindowPos(tooltips_window, HWND_TOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE|SWP_NOSENDCHANGING);

	return TRUE;
}

void destroyTooltipsWindow(void)
{
	deleteTooltip();

	if ( tooltips_window )
		DestroyWindow(tooltips_window);

	tooltips_window = NULL;
}
