/*
  This file is part of KMHOOK.DLL source code
  KMHOOK.DLL is native Win32 library providing system-wide hooks

  Copyright (C) 2000-2001 Pavel Vitis
  pavel.vitis@seznam.cz
  http://lsg.come.to/

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <windows.h>
#include <stdlib.h>
#include "../kmhook/hookdll.h"


//#define BE_SAFE

UINT MouseMsg = 0;
UINT KeyboardMsg = 0;
UINT ShellMsg = 0;
HMODULE hLibHookLib = NULL;

GETMSGPROC getMonitorKeyboardMsg, getMonitorMouseMsg, getMonitorShellMsg;
SETHOOKPROC setMouseHook, setKeyboardHook, setShellHook;
UNSETHOOKPROC unsetMouseHook, unsetKeyboardHook, unsetShellHook;
BOOLPROC isHookSet;
WORDPROC getHookDLLVersion;

#ifdef BE_SAFE
// Define some dummy functions for case of DLL laoding error

__cdecl WORD dummyGetDLLVersion( void )
{
	return 0x0100;
}

__cdecl BOOL dummySetHook( HWND wnd, EVENTS evnt )
{
	return FALSE;
}

__cdecl BOOL dummyUnsetHook( HWND wnd )
{
	return FALSE;
}

__cdecl BOOL dummyIsHookSet( void )
{
	return FALSE;
}

__cdecl UINT dummyGetMsg( void )
{
	return 0;
}
#endif // BE_SAFE

// Dynamically load hook dll into address space

BOOL loadHookDLL( HINSTANCE hInstance )
{
	char DLLPath[MAX_PATH+1];
	char DLLDrive[_MAX_DRIVE+1];
	char DLLDir[_MAX_DIR+1];

	// Try to get hook dll handle, if already in memory
	hLibHookLib = GetModuleHandle(HOOK_DLL);
	if ( hLibHookLib )
	{
		#ifdef HOOK_LOG
		GetModuleFileName( hLibHookLib, DLLPath, MAX_PATH );
		OutputDebugString( DLLPath );
		#endif
	}
	else
	{
		// Load it from disk
		GetModuleFileName( hInstance, DLLPath, sizeof(DLLPath) );
		_splitpath( DLLPath, DLLDrive, DLLDir, NULL, NULL );
		strcpy( DLLPath, DLLDrive );
		strcat( DLLPath, DLLDir );
		strcat( DLLPath, HOOK_DLL );
		#ifdef HOOK_LOG
		OutputDebugString( DLLPath );
		#endif
		hLibHookLib = LoadLibrary(DLLPath);
	}

	if ( hLibHookLib )
    {
		// Get pointers to functions

		getMonitorMouseMsg = (GETMSGPROC) GetProcAddress( hLibHookLib, "getMonitorMouseMsg");
		if ( getMonitorMouseMsg != NULL )
		{
			MouseMsg = (getMonitorMouseMsg)();
			#ifdef HOOK_LOG
			OutputDebugString( itoa(MouseMsg, msgbuf, 10) );
			#endif
		}
#ifdef BE_SAFE
		else
			getMonitorMouseMsg = dummyGetMsg;
#endif // BE_SAFE

        getMonitorKeyboardMsg = (GETMSGPROC) GetProcAddress(hLibHookLib, "getMonitorKeyboardMsg");
        if ( getMonitorKeyboardMsg != NULL )
		{
			KeyboardMsg = (getMonitorKeyboardMsg)();
			#ifdef HOOK_LOG
			OutputDebugString( itoa(KeyboardMsg, msgbuf, 10) );
			#endif
		}
#ifdef BE_SAFE
		else
			getMonitorKeyboardMsg = dummyGetMsg;
#endif // BE_SAFE

        getMonitorShellMsg = (GETMSGPROC) GetProcAddress(hLibHookLib, "getMonitorShellMsg");
        if ( getMonitorShellMsg != NULL )
		{
			ShellMsg = (getMonitorShellMsg)();
			#ifdef HOOK_LOG
			OutputDebugString( itoa(ShellMsg, msgbuf, 10) );
			#endif
		}
#ifdef BE_SAFE
		else
			getMonitorShellMsg = dummyGetMsg;
#endif // BE_SAFE

		setKeyboardHook = (SETHOOKPROC) GetProcAddress(hLibHookLib, "setKeyboardHook");
#ifdef BE_SAFE
		if ( setKeyboardHook == NULL )
			setKeyboardHook = dummySetHook;
#endif // BE_SAFE

        unsetKeyboardHook = (UNSETHOOKPROC) GetProcAddress(hLibHookLib, "unsetKeyboardHook");
#ifdef BE_SAFE
		if ( unsetKeyboardHook == NULL )
			unsetKeyboardHook = dummyUnsetHook;
#endif // BE_SAFE

		setMouseHook = (SETHOOKPROC) GetProcAddress(hLibHookLib, "setMouseHook");
#ifdef BE_SAFE
		if ( setMouseHook == NULL )
			setMouseHook = dummySetHook;
#endif // BE_SAFE

        unsetMouseHook = (UNSETHOOKPROC) GetProcAddress(hLibHookLib, "unsetMouseHook");
#ifdef BE_SAFE
		if ( unsetMouseHook == NULL )
			unsetMouseHook = dummyUnsetHook;
#endif // BE_SAFE

		setShellHook = (SETHOOKPROC) GetProcAddress(hLibHookLib, "setShellHook");
#ifdef BE_SAFE
		if ( setShellHook == NULL )
			setShellHook = dummySetHook;
#endif // BE_SAFE

        unsetShellHook = (UNSETHOOKPROC) GetProcAddress(hLibHookLib, "unsetShellHook");
#ifdef BE_SAFE
		if ( unsetShellHook == NULL )
			unsetShellHook = dummyUnsetHook;
#endif // BE_SAFE

		isHookSet = (BOOLPROC) GetProcAddress(hLibHookLib, "isHookSet");
#ifdef BE_SAFE
		if ( isHookSet == NULL )
			isHookSet = dummyIsHookSet;
#endif // BE_SAFE

		getHookDLLVersion = (WORDPROC) GetProcAddress(hLibHookLib, "getDLLVersion");
#ifdef BE_SAFE
		if ( getHookDLLVersion == NULL )
			getHookDLLVersion = dummyGetDLLVersion;
#endif // BE_SAFE

	}
	else
		MessageBox(NULL, "Could not load", HOOK_DLL TEXT(" error"), MB_OK | MB_ICONEXCLAMATION | MB_TOPMOST);

	return (hLibHookLib != NULL);
}

void unloadHookDLL( void )
{
	if ( !isHookSet() )
	{
		FreeLibrary(hLibHookLib);
		#ifdef HOOK_LOG
    	OutputDebugString( HOOK_DLL TEXT(" Library unloaded") );
		#endif
	}
}

